@@ -8,11 +8,14 @@ import functools
 import inspect
 import re
 import sys
+from contextlib import contextmanager
 
 import py
 
 import _pytest
 from _pytest.outcomes import TEST_OUTCOME
+from six import text_type
+import six
 
 try:
     import enum
@@ -20,7 +23,6 @@ except ImportError:  # pragma: no cover
     # Only available in Python 3.4+ or as a backport
     enum = None
 
-
 _PY3 = sys.version_info > (3, 0)
 _PY2 = not _PY3
 
@@ -30,23 +32,31 @@ if _PY3:
 else:
     from funcsigs import signature, Parameter as Parameter
 
-
 NoneType = type(None)
 NOTSET = object()
 
 PY35 = sys.version_info[:2] >= (3, 5)
 PY36 = sys.version_info[:2] >= (3, 6)
-MODULE_NOT_FOUND_ERROR = 'ModuleNotFoundError' if PY36 else 'ImportError'
+MODULE_NOT_FOUND_ERROR = "ModuleNotFoundError" if PY36 else "ImportError"
+
 
 if _PY3:
-    from collections.abc import MutableMapping as MappingMixin  # noqa
-    from collections.abc import Mapping, Sequence  # noqa
+    from collections.abc import MutableMapping as MappingMixin
+    from collections.abc import Mapping, Sequence
 else:
     # those raise DeprecationWarnings in Python >=3.7
     from collections import MutableMapping as MappingMixin  # noqa
     from collections import Mapping, Sequence  # noqa
 
 
+if sys.version_info >= (3, 4):
+    from importlib.util import spec_from_file_location
+else:
+
+    def spec_from_file_location(*_, **__):
+        return None
+
+
 def _format_args(func):
     return str(signature(func))
 
@@ -54,9 +64,9 @@ def _format_args(func):
 isfunction = inspect.isfunction
 isclass = inspect.isclass
 # used to work around a python2 exception info leak
-exc_clear = getattr(sys, 'exc_clear', lambda: None)
+exc_clear = getattr(sys, "exc_clear", lambda: None)
 # The type of re.compile objects is not exposed in Python.
-REGEX_TYPE = type(re.compile(''))
+REGEX_TYPE = type(re.compile(""))
 
 
 def is_generator(func):
@@ -70,13 +80,15 @@ def iscoroutinefunction(func):
     Note: copied and modified from Python 3.5's builtin couroutines.py to avoid import asyncio directly,
     which in turns also initializes the "logging" module as side-effect (see issue #8).
     """
-    return (getattr(func, '_is_coroutine', False) or
-            (hasattr(inspect, 'iscoroutinefunction') and inspect.iscoroutinefunction(func)))
+    return getattr(func, "_is_coroutine", False) or (
+        hasattr(inspect, "iscoroutinefunction") and inspect.iscoroutinefunction(func)
+    )
 
 
 def getlocation(function, curdir):
+    function = get_real_func(function)
     fn = py.path.local(inspect.getfile(function))
-    lineno = py.builtin._getcode(function).co_firstlineno
+    lineno = function.__code__.co_firstlineno
     if fn.relto(curdir):
         fn = fn.relto(curdir)
     return "%s:%d" % (fn, lineno + 1)
@@ -90,8 +102,9 @@ def num_mock_patch_args(function):
     mock_modules = [sys.modules.get("mock"), sys.modules.get("unittest.mock")]
     if any(mock_modules):
         sentinels = [m.DEFAULT for m in mock_modules if m is not None]
-        return len([p for p in patchings
-                    if not p.attribute_name and p.new in sentinels])
+        return len(
+            [p for p in patchings if not p.attribute_name and p.new in sentinels]
+        )
     return len(patchings)
 
 
@@ -118,47 +131,65 @@ def getfuncargnames(function, is_method=False, cls=None):
     # ordered mapping of parameter names to Parameter instances.  This
     # creates a tuple of the names of the parameters that don't have
     # defaults.
-    arg_names = tuple(p.name for p in signature(function).parameters.values()
-                      if (p.kind is Parameter.POSITIONAL_OR_KEYWORD or
-                          p.kind is Parameter.KEYWORD_ONLY) and
-                      p.default is Parameter.empty)
+    arg_names = tuple(
+        p.name
+        for p in signature(function).parameters.values()
+        if (
+            p.kind is Parameter.POSITIONAL_OR_KEYWORD
+            or p.kind is Parameter.KEYWORD_ONLY
+        )
+        and p.default is Parameter.empty
+    )
     # If this function should be treated as a bound method even though
     # it's passed as an unbound method or function, remove the first
     # parameter name.
-    if (is_method or
-        (cls and not isinstance(cls.__dict__.get(function.__name__, None),
-                                staticmethod))):
+    if is_method or (
+        cls and not isinstance(cls.__dict__.get(function.__name__, None), staticmethod)
+    ):
         arg_names = arg_names[1:]
     # Remove any names that will be replaced with mocks.
     if hasattr(function, "__wrapped__"):
-        arg_names = arg_names[num_mock_patch_args(function):]
+        arg_names = arg_names[num_mock_patch_args(function) :]
     return arg_names
 
 
+@contextmanager
+def dummy_context_manager():
+    """Context manager that does nothing, useful in situations where you might need an actual context manager or not
+    depending on some condition. Using this allow to keep the same code"""
+    yield
+
+
 def get_default_arg_names(function):
     # Note: this code intentionally mirrors the code at the beginning of getfuncargnames,
     # to get the arguments which were excluded from its result because they had default values
-    return tuple(p.name for p in signature(function).parameters.values()
-                 if p.kind in (Parameter.POSITIONAL_OR_KEYWORD, Parameter.KEYWORD_ONLY) and
-                 p.default is not Parameter.empty)
+    return tuple(
+        p.name
+        for p in signature(function).parameters.values()
+        if p.kind in (Parameter.POSITIONAL_OR_KEYWORD, Parameter.KEYWORD_ONLY)
+        and p.default is not Parameter.empty
+    )
 
 
 if _PY3:
     STRING_TYPES = bytes, str
-    UNICODE_TYPES = str,
+    UNICODE_TYPES = six.text_type
 
     if PY35:
+
         def _bytes_to_ascii(val):
-            return val.decode('ascii', 'backslashreplace')
+            return val.decode("ascii", "backslashreplace")
+
     else:
+
         def _bytes_to_ascii(val):
             if val:
                 # source: http://goo.gl/bGsnwC
                 encoded_bytes, _ = codecs.escape_encode(val)
-                return encoded_bytes.decode('ascii')
+                return encoded_bytes.decode("ascii")
             else:
                 # empty bytes crashes codecs.escape_encode (#1087)
-                return ''
+                return ""
 
     def ascii_escaped(val):
         """If val is pure ascii, returns it as a str().  Otherwise, escapes
@@ -181,10 +212,12 @@ if _PY3:
         if isinstance(val, bytes):
             return _bytes_to_ascii(val)
         else:
-            return val.encode('unicode_escape').decode('ascii')
+            return val.encode("unicode_escape").decode("ascii")
+
+
 else:
-    STRING_TYPES = bytes, str, unicode
-    UNICODE_TYPES = unicode,
+    STRING_TYPES = six.string_types
+    UNICODE_TYPES = six.text_type
 
     def ascii_escaped(val):
         """In py2 bytes and str are the same type, so return if it's a bytes
@@ -197,11 +230,23 @@ else:
         """
         if isinstance(val, bytes):
             try:
-                return val.encode('ascii')
+                return val.encode("ascii")
             except UnicodeDecodeError:
-                return val.encode('string-escape')
+                return val.encode("string-escape")
         else:
-            return val.encode('unicode-escape')
+            return val.encode("unicode-escape")
+
+
+class _PytestWrapper(object):
+    """Dummy wrapper around a function object for internal use only.
+
+    Used to correctly unwrap the underlying function object
+    when we are creating fixtures, because we wrap the function object ourselves with a decorator
+    to issue warnings when the fixture function is called directly.
+    """
+
+    def __init__(self, obj):
+        self.obj = obj
 
 
 def get_real_func(obj):
@@ -210,25 +255,47 @@ def get_real_func(obj):
     """
     start_obj = obj
     for i in range(100):
-        new_obj = getattr(obj, '__wrapped__', None)
+        # __pytest_wrapped__ is set by @pytest.fixture when wrapping the fixture function
+        # to trigger a warning if it gets called directly instead of by pytest: we don't
+        # want to unwrap further than this otherwise we lose useful wrappings like @mock.patch (#3774)
+        new_obj = getattr(obj, "__pytest_wrapped__", None)
+        if isinstance(new_obj, _PytestWrapper):
+            obj = new_obj.obj
+            break
+        new_obj = getattr(obj, "__wrapped__", None)
         if new_obj is None:
             break
         obj = new_obj
     else:
         raise ValueError(
-            ("could not find real function of {start}"
-             "\nstopped at {current}").format(
-                start=py.io.saferepr(start_obj),
-                current=py.io.saferepr(obj)))
+            ("could not find real function of {start}\nstopped at {current}").format(
+                start=py.io.saferepr(start_obj), current=py.io.saferepr(obj)
+            )
+        )
     if isinstance(obj, functools.partial):
         obj = obj.func
     return obj
 
 
+def get_real_method(obj, holder):
+    """
+    Attempts to obtain the real function object that might be wrapping ``obj``, while at the same time
+    returning a bound method to ``holder`` if the original object was a bound method.
+    """
+    try:
+        is_method = hasattr(obj, "__func__")
+        obj = get_real_func(obj)
+    except Exception:
+        return obj
+    if is_method and hasattr(obj, "__get__") and callable(obj.__get__):
+        obj = obj.__get__(holder)
+    return obj
+
+
 def getfslineno(obj):
     # xxx let decorators etc specify a sane ordering
     obj = get_real_func(obj)
-    if hasattr(obj, 'place_as'):
+    if hasattr(obj, "place_as"):
         obj = obj.place_as
     fslineno = _pytest._code.getfslineno(obj)
     assert isinstance(fslineno[1], int), obj
@@ -267,39 +334,44 @@ def _is_unittest_unexpected_success_a_failure():
 
 
 if _PY3:
+
     def safe_str(v):
         """returns v as string"""
         return str(v)
+
+
 else:
+
     def safe_str(v):
         """returns v as string, converting to ascii if necessary"""
         try:
             return str(v)
         except UnicodeError:
-            if not isinstance(v, unicode):
-                v = unicode(v)
-            errors = 'replace'
-            return v.encode('utf-8', errors)
+            if not isinstance(v, text_type):
+                v = text_type(v)
+            errors = "replace"
+            return v.encode("utf-8", errors)
 
 
 COLLECT_FAKEMODULE_ATTRIBUTES = (
-    'Collector',
-    'Module',
-    'Generator',
-    'Function',
-    'Instance',
-    'Session',
-    'Item',
-    'Class',
-    'File',
-    '_fillfuncargs',
+    "Collector",
+    "Module",
+    "Generator",
+    "Function",
+    "Instance",
+    "Session",
+    "Item",
+    "Class",
+    "File",
+    "_fillfuncargs",
 )
 
 
 def _setup_collect_fakemodule():
     from types import ModuleType
     import pytest
-    pytest.collect = ModuleType('pytest.collect')
+
+    pytest.collect = ModuleType("pytest.collect")
     pytest.collect.__all__ = []  # used for setns
     for attr in COLLECT_FAKEMODULE_ATTRIBUTES:
         setattr(pytest.collect, attr, getattr(pytest, attr))
@@ -310,10 +382,10 @@ if _PY2:
     from py.io import TextIO
 
     class CaptureIO(TextIO):
-
         @property
         def encoding(self):
-            return getattr(self, '_encoding', 'UTF-8')
+            return getattr(self, "_encoding", "UTF-8")
+
 
 else:
     import io
@@ -321,18 +393,18 @@ else:
     class CaptureIO(io.TextIOWrapper):
         def __init__(self):
             super(CaptureIO, self).__init__(
-                io.BytesIO(),
-                encoding='UTF-8', newline='', write_through=True,
+                io.BytesIO(), encoding="UTF-8", newline="", write_through=True
             )
 
         def getvalue(self):
-            return self.buffer.getvalue().decode('UTF-8')
+            return self.buffer.getvalue().decode("UTF-8")
 
 
 class FuncargnamesCompatAttr(object):
     """ helper class so that Metafunc, Function and FixtureRequest
     don't need to each define the "funcargnames" compatibility attribute.
     """
+
     @property
     def funcargnames(self):
         """ alias attribute for ``fixturenames`` for pre-2.3 compatibility"""
